/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef __COMPONENTTREE_H__
#define __COMPONENTTREE_H__

#include <vector>
#include <map>
#include "misc.h"

/**
 * Creates and holds a component tree.
 */
class ComponentTree {
	public:
		/**
		 * A node in the component tree.
		 */
		struct Node {
			Node(Node* parent, Component comp);
			~Node();

			Node* _parent;
			Component _comp;
			std::vector<Node*> _children;
		};

		/**
		 * Creates a component tree from a list of components.
		 */
		ComponentTree(const std::vector<Component>& components);

		~ComponentTree();

		/**
		 * Transforms the tree into the minimal tree containing all unoriented componentes.
		 * \todo come up with a better name
		 */
		void makeUnoriented();

		/**
		 * Count the number of leaves in the component tree.
		 * This is the number of hurdles, if makeUnoriented has been called.
		 */
		size_t countLeaves();

		/**
		 * Number of short branches.
		 * If makeUnoriented has been called and countLeaves is >= 3
		 * then we have a super hurdle if short branches = 0.
		 */
		size_t shortBranches();

	private:
		//Disable these, at least for now.
		void operator=(const ComponentTree&){};
		ComponentTree(const ComponentTree&): _root(0){};

		void removeOriented(Node* n);
		size_t countLeaves(Node* n);
		void branches (Node* n, std::map<Node*,size_t> & b);

		Node* _root;

	friend class ComponentTreeTest;
};

#endif
