/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef __MODEL_H__
#define __MODEL_H__

#include "shared_ptr.h"
#include <string>

class SortAction;
class GeneOrder;

namespace Models {
	class ModelImpl;
}

/**
 * Abstraction of a model.
 * 
 * Handles scoring of specific actions on gene orders.
 */
class Model{
	public:
		typedef shared_ptr<Models::ModelImpl> ModelPointer;

		/**
		 * Constructs a new model.
		 *
		 * \param model The actual model implementation. This model takes 
		 * 	ownership of the new allocated implementation and will delete it.
		 */
		Model(Models::ModelImpl* model);

		/**
		 * Gives the score for the action on a given gene order.
		 * A better match for this model gives a higher score.
		 */
		double score(const SortAction& sa, const GeneOrder& go) const;

		/**
		 * The name for the model used.
		 */
		const char* name() const;

		/**
		 * Creates a model from a name.
		 */
		static Model modelFactory(const std::string& name);

	private:
		ModelPointer _impl;

};

bool operator<(const Model& lh,const Model& rh);

#endif
