/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef __MODELS_H__
#define __MODELS_H__

class SortAction;
class GeneOrder;

namespace Models {

/**
 * Basic implementation of a model. All sort actions are considered equal.
 */
class ModelImpl {
	public:
		virtual ~ModelImpl();

		/**
		 * All actions return 1 in this model.
		 *
		 * \see Model::score
		 */
		virtual double score(const SortAction& sa, const GeneOrder& go);

		/**
		 * \see Model::name
		 */
		virtual const char* name();
};

/**
 * A model that favors reversals.
 */
class Whirl : public ModelImpl {
	public:
		/**
		 * Returns a higher score for reversals and a lower score
		 * for all other actions.
		 *
		 * \see Model::score
		 */
		double score(const SortAction& sa, const GeneOrder& go);

		/**
		 * \see Model::name
		 */
		const char* name();
};

/**
 * A model that favors symmetric reversal, resulting in an X-like
 * picture in a dot plot.
 */
class X : public ModelImpl {
	public:
		/**
		 * Returns a higher score for reversals around the center
		 *
		 * \see Model::score
		 */
		double score(const SortAction& sa, const GeneOrder& go);

		/**
		 * \see Model::name
		 */
		const char* name();
};

/**
 * A model that favors short reversals, resulting in a zipper-like
 * picture in a dot plot
 */
class Zipper : public ModelImpl {
	public:
		/**
		 * Returns higher score for shorter reversals.
		 *
		 * \see Model::score
		 */
		double score(const SortAction& sa, const GeneOrder& go);

		/**
		 * \see Model::name
		 */
		const char* name();
};

/**
 * A model that favors single gene transpositions, resulting in a
 * very scattered dot plot.
 */
class Cloud : public ModelImpl {
	public:
		/**
		 * \todo has to be implemented
		 * \see Model::score
		 */
		double score(const SortAction& sa, const GeneOrder& go);

		/**
		 * \see Model::name
		 */
		const char* name();
};

/**
 * A model similar to X, but a wider X in the dot plot.
 */
class FatX : public ModelImpl {
	public:
		/**
		 * \todo has to be implemented
		 * \see Model::score
		 */
		double score(const SortAction& sa, const GeneOrder& go);

		/**
		 * \see Model::name
		 */
		const char* name();
};

}

#endif
